@@ -57,9 +57,20 @@ from mercurial import (
 )
 
 from mercurial.windows import posixfile, unlink, rename
-from tortoisehg.util.i18n import agettext as _
+from tortoisehg.util.i18n import _
 from tortoisehg.util import hglib, thread2, paths, shlib, version
 
+if hglib.TYPE_CHECKING:
+    from typing import (
+        Any,
+        Iterable,
+        List,
+        Set,
+        Text,
+        Tuple,
+    )
+
+
 APP_TITLE = _('TortoiseHg Overlay Icon Server')
 
 EXIT_CMD = 1025
@@ -69,6 +80,7 @@ class Logger:
         self.file = None
 
     def setfile(self, name):
+        # type: (Text) -> None
         oname = name + '.old'
         try:
             rename(name, oname)
@@ -79,10 +91,11 @@ class Logger:
         self.msg('Logging to file started')
 
     def msg(self, msg):
+        # type: (Text) -> None
         ts = '[%s] ' % time.strftime('%c')
         f = self.file
         if f:
-            f.write(ts + msg + '\n')
+            f.write(hglib.fromunicode(ts + msg) + b'\n')
             f.flush()
             os.fsync(f.fileno())
             print('L' + ts + msg)
@@ -94,6 +107,7 @@ logger = Logger()
 iconcache = {}
 
 def load_icon(name):
+    # type: (Text) -> Any
     from tortoisehg.util.paths import get_tortoise_icon
     iconPathName = get_tortoise_icon(name)
     if iconPathName and os.path.isfile(iconPathName):
@@ -105,6 +119,7 @@ def load_icon(name):
     return hicon
 
 def get_icon(name):
+    # type: (Text) -> Any
     try:
         return iconcache[name]
     except KeyError:
@@ -112,6 +127,7 @@ def get_icon(name):
         return iconcache[name]
 
 def SetIcon(hwnd, name, add=False):
+    # type: (Any, Text, bool) -> None
     # Try and find a custom icon
     if '--noicon' in sys.argv:
         return
@@ -210,7 +226,7 @@ class MainWindow:
 
     def stop_pipe_server(self):
         logger.msg("MainWindow.stop_pipe_server")
-        if not self.pipethread.isAlive():
+        if not self.pipethread.is_alive():
             logger.msg("pipethread is not alive")
             return True
 
@@ -219,20 +235,20 @@ class MainWindow:
 
         max_try = 10
         cnt = 1
-        while cnt <= max_try and self.pipethread.isAlive():
+        while cnt <= max_try and self.pipethread.is_alive():
             print("testing pipe [try %d] ..." % cnt)
             try:
                 try:
                     self.pipethread.terminate()
                 except ValueError:
                     pass
-                win32pipe.CallNamedPipe(PIPENAME, '', PIPEBUFSIZE, 0)
+                win32pipe.CallNamedPipe(PIPENAME, b'', PIPEBUFSIZE, 0)
             except:
                 logger.msg(traceback.format_exc())
                 pass
             cnt += 1
 
-        if self.pipethread.isAlive():
+        if self.pipethread.is_alive():
             msg = "WARNING: unable to stop server after %d trys." % max_try
             logger.msg(msg)
             return False
@@ -257,6 +273,7 @@ PIPENAME += GetUserName()
 PIPEBUFSIZE = 4096
 
 def getrepos(batch):
+    # type: (Iterable[Text]) -> Tuple[Set[Text], Set[Text]]
     roots = set()
     notifypaths = set()
     for path in batch:
@@ -278,6 +295,7 @@ def getrepos(batch):
     return roots, notifypaths
 
 def update_batch(batch):
+    # type: (Iterable[Text]) -> None
     '''updates thgstatus for all paths in batch'''
     roots, notifypaths = getrepos(batch)
     if roots:
@@ -296,9 +314,11 @@ def update_batch(batch):
             updated_any = False
             for r in sorted(roots):
                 try:
-                    if shlib.update_thgstatus(_ui, r, wait=False):
+                    if shlib.update_thgstatus(
+                        _ui, hglib.fromunicode(r), wait=False
+                    ):
                         updated_any = True
-                    shlib.shell_notify([r], noassoc=True)
+                    shlib.shell_notify([hglib.fromunicode(r)], noassoc=True)
                     logger.msg('Updated ' + r)
                 except (IOError, OSError):
                     print("IOError or OSError on updating %s (check permissions)" % r)
@@ -310,7 +330,10 @@ def update_batch(batch):
                     failedroots.add(r)
             notifypaths -= failedroots
             if notifypaths:
-                shlib.shell_notify(list(notifypaths), noassoc=not updated_any)
+                shlib.shell_notify(
+                    [hglib.fromunicode(p) for p in notifypaths],
+                    noassoc=not updated_any
+                )
                 logger.msg('Shell notified')
             if not pycompat.ispy3:
                 errmsg = errorstream.getvalue()
@@ -363,6 +386,7 @@ def get_config():
     return show_taskbaricon, hgighlight_taskbaricon
 
 def update(args, hwnd):
+    # type: (List[Text], Any) -> None
     batch = []
     r = args[0]
     print("got update request %s (first in batch)" % r)
@@ -394,6 +418,7 @@ def update(args, hwnd):
         SetIcon(hwnd, "hg.ico")
 
 def remove(args):
+    # type: (List[Text]) -> None
     path = args[0]
     logger.msg('Removing ' + path)
     roots, notifypaths = getrepos([path])
@@ -404,15 +429,16 @@ def remove(args):
                 f = posixfile(tfn, b'rb')
                 e = f.readline()
                 f.close()
-                if not e.startswith('@@noicons'):
+                if not e.startswith(b'@@noicons'):
                     unlink(tfn)
             except (IOError, OSError) as e:
                 if e.errno != errno.ENOENT:
                     logger.msg("Error while trying to remove %s (%s)" % (tfn, e))
         if notifypaths:
-            shlib.shell_notify(list(notifypaths))
+            shlib.shell_notify([hglib.fromunicode(p) for p in notifypaths])
 
 def dispatch(req, cmd, args, hwnd):
+    # type: (Any, Text, List[Text], Any) -> None
     print("dispatch(%s)" % req)
     if cmd == 'update':
         update(args, hwnd)
@@ -515,7 +541,7 @@ class PipeServer:
                     continue
 
                 try:
-                    requests.put(data)
+                    requests.put(hglib.tounicode(data))
                     if data == 'terminate|':
                         logger.msg('PipeServer received terminate from pipe')
                         PostMessage(self.hwnd, win32con.WM_COMMAND, EXIT_CMD, 0)
