@@ -80,6 +80,10 @@ if [ -d dist/${APP_NAME}.app ]; then
         rm -r "${f}"
     done
 
+    # Qt5 doesn't support ARM64, so strip the image to force the app to run
+    # under Rosetta.
+    xcrun lipo -thin x86_64 -o "${APP}/Contents/MacOS/TortoiseHg" "${APP}/Contents/MacOS/TortoiseHg"
+
     if [ -z "${CODE_SIGN_IDENTITY-}" ]; then
         # These ad hoc signing parameters are copied from py2app/util._dosign()
         CODE_SIGN_IDENTITY="-"
